@@ -3,7 +3,7 @@ App::DBBrowser::Table::Extensions::ScalarFunctions::Numeric;
 
 use warnings;
 use strict;
-use 5.016;
+use 5.014;
 
 use App::DBBrowser::Auxil;
 
@@ -20,9 +20,9 @@ sub new {
 
 sub function_rand {
     my ( $sf ) = @_;
-    my $dbms = $sf->{i}{dbms};
-    return "RANDOM()"          if $dbms =~ /^(?:SQLite|Pg|DuckDB)\z/;
-    return "DBMS_RANDOM.VALUE" if $dbms eq 'Oracle';
+    my $driver = $sf->{i}{driver};
+    return "RANDOM()"          if $driver =~ /^(?:SQLite|Pg)\z/;
+    return "DBMS_RANDOM.VALUE" if $driver eq 'Oracle';
     return "RAND()";
 }
 
@@ -47,25 +47,20 @@ sub function_truncate {
 
 sub __round_trunc_truncate {
     my ( $sf, $sql, $clause, $func, $cols, $r_data ) = @_;
-    my $dbms = $sf->{i}{dbms};
+    my $driver = $sf->{i}{driver};
     my $ga = App::DBBrowser::Table::Extensions::ScalarFunctions::GetArguments->new( $sf->{i}, $sf->{o}, $sf->{d} );
     my $col = $ga->choose_a_column( $sql, $clause, $cols, $r_data );
     if ( ! defined $col ) {
         return;
     }
-    if ( $dbms eq 'DuckDB' && $func eq 'TRUNC' ) {
-        return "$func($col)";
-    }
     my $args_data = [
         { prompt => 'Decimal places: ', is_numeric => 1 }
     ];
     my ( $places ) = $ga->get_arguments( $sql, $clause, $func, $args_data, $r_data );
     if ( ! defined $places ) {
-        return if $func eq 'TRUNCATE' && $dbms =~ /^(?:mysql|MariaDB)\z/;
-        return if $dbms eq 'MSSQL';
+        return if $func eq 'TRUNCATE' && $driver =~ /^(?:mysql|MariaDB)\z/;
         return "$func($col)";
     }
-    return "ROUND($col,$places,1)" if $func eq 'TRUNC' && $dbms eq 'MSSQL';
     return "$func($col,$places)";
 }
 
@@ -73,7 +68,6 @@ sub __round_trunc_truncate {
 sub function_mod {
     my ( $sf, $sql, $clause, $func, $cols, $r_data ) = @_;
     my $ga = App::DBBrowser::Table::Extensions::ScalarFunctions::GetArguments->new( $sf->{i}, $sf->{o}, $sf->{d} );
-    my $dbms = $sf->{i}{dbms};
     my $col = $ga->choose_a_column( $sql, $clause, $cols, $r_data );
     if ( ! defined $col ) {
         return;
@@ -83,7 +77,6 @@ sub function_mod {
     ];
     my ( $divider ) = $ga->get_arguments( $sql, $clause, $func, $args_data, $r_data );
     return if ! defined $divider;
-    return "($col % $divider)" if $dbms eq 'MSSQL'; # ###
     return "MOD($col,$divider)";
 }
 
