@@ -3,7 +3,7 @@ App::DBBrowser::Table::Extensions::ScalarFunctions::String;
 
 use warnings;
 use strict;
-use 5.016;
+use 5.014;
 
 use App::DBBrowser::Auxil;
 use App::DBBrowser::Table::Extensions::ScalarFunctions::GetArguments;
@@ -107,9 +107,6 @@ sub __rpad_lpad {
     if ( ! defined $length ) {
         return;
     }
-    if ( $driver eq 'DuckDB' ) {
-        $fill = length $fill ? $fill : "' '";
-    }
     if ( $driver eq 'SQLite' ) {
         $fill = defined $fill ? $ax->unquote_constant( $fill ) : ' ';
         $fill = $sf->{d}{dbh}->quote( $fill x $length );
@@ -178,6 +175,9 @@ sub function_instr {
     if ( ! defined $substring ) {
         return;
     }
+    if ( $driver eq 'SQLite' ) {
+        return "INSTR($col,$substring)";
+    }
     return "INSTR($col,$substring)"                if ! defined $start;
     return "INSTR($col,$substring,$start)"         if ! defined $count;
     return "INSTR($col,$substring,$start,$count)";
@@ -265,7 +265,7 @@ sub function_trim {
     }
     else {
         $args_data = [
-            { prompt => 'Where: ', unquote => 1, history => [ qw(BOTH LEADING TRAILING) ], history_only => 1, skip_ok => 1 },
+            { prompt => 'Where: ', history => [ qw(BOTH LEADING TRAILING) ], history_only => 1, unquote => 1, skip_ok => 1 },
             { prompt => 'What: ' }
         ];
     }
@@ -298,7 +298,7 @@ sub function_concat {
         }
     }
     my $args_data = [
-        { prompt => 'Separator: ', history => [ '-', ' ', '_', ',', '/', '=', '+', '|' ] },
+        { prompt => 'Separator: ', history => [ '-', ' ', '_', ',', '/', '=', '+' ] },
     ];
     my ( $sep ) = $ga->get_arguments( $sql, $clause, $func, $args_data, $r_data );
     my $arg;
