@@ -3,7 +3,7 @@ App::DBBrowser::Table::Extensions::ScalarFunctions::To::EpochTo;
 
 use warnings;
 use strict;
-use 5.016;
+use 5.014;
 
 use Scalar::Util qw( looks_like_number );
 
@@ -77,17 +77,14 @@ sub __select_stmt {
     else {
         $stmt = "SELECT $select_col FROM $sql->{table} WHERE $where_col IS NOT NULL";
     }
-    if ( $sql->{order_by_stmt} ) {
-        $stmt .= " " . $sql->{order_by_stmt};
-    }
-    if ( $sql->{limit_stmt} =~ /^LIMIT\b/i ) {
-        $stmt .= " " . $sql->{limit_stmt};
-        $stmt .= " " . $sql->{offset_stmt} if $sql->{offset_stmt};
-    }
-    else {
+    if ( $sf->{i}{driver} =~ /^(?:Firebird|DB2|Oracle)\z/ ) {
         $stmt .= " " . $sql->{offset_stmt} if $sql->{offset_stmt};
         $stmt .= " " . $sql->{limit_stmt}  if $sql->{limit_stmt};
     }
+    else {
+        $stmt .= " " . $sql->{limit_stmt}  if $sql->{limit_stmt};
+        $stmt .= " " . $sql->{offset_stmt} if $sql->{offset_stmt};
+    }
     return $stmt;
 }
 
@@ -224,17 +221,17 @@ sub __choose_interval {
 sub __stmt_epoch_to_date {
     my ( $sf, $col, $interval ) = @_;
     #my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
-    my $dbms = $sf->{i}{dbms};
-    if ( $dbms eq 'SQLite' ) {
+    my $driver = $sf->{i}{driver};
+    if ( $driver eq 'SQLite' ) {
         return "DATE($col/$interval,'unixepoch','localtime')";
     }
-    elsif ( $dbms =~ /^(?:mysql|MariaDB)\z/ ) {
+    elsif ( $driver =~ /^(?:mysql|MariaDB)\z/ ) {
         return "FROM_UNIXTIME($col/$interval,'%Y-%m-%d')";
     }
-    elsif ( $dbms =~ /^(?:Pg|DuckDB)\z/ ) { ##
+    elsif ( $driver eq 'Pg' ) {
         return "TO_TIMESTAMP(${col}::bigint/$interval)::date";
     }
-    elsif ( $dbms eq 'Firebird' ) {
+    elsif ( $driver eq 'Firebird' ) {
         #my $firebird_major_version = $ax->major_server_version();
         my $firebird_major_version = 3; ##
         if ( $firebird_major_version >= 4 ) {
@@ -246,31 +243,27 @@ sub __stmt_epoch_to_date {
             return "CAST(DATEADD(CAST($col AS BIGINT)/$interval SECOND TO DATE '1970-01-01') AS VARCHAR(10))";
         }
     }
-    elsif ( $dbms eq 'DB2' ) {
+    elsif ( $driver eq 'DB2' ) {
         return "TIMESTAMP('1970-01-01') + ($col/$interval) SECONDS";
     }
-    elsif ( $dbms eq 'Informix' ) {
+    elsif ( $driver eq 'Informix' ) {
         return "TO_CHAR(DBINFO('utc_to_datetime',$col/$interval),'%Y-%m-%d')";
     }
-    elsif ( $dbms eq 'Oracle' ) {
+    elsif ( $driver eq 'Oracle' ) {
         return "TO_CHAR((TIMESTAMP '1970-01-01 00:00:00 UTC' + $col/$interval * INTERVAL '1' SECOND) AT TIME ZONE SESSIONTIMEZONE,'YYYY-MM-DD')";
     }
-    elsif ( $dbms eq 'MSSQL' ) { # no timezone
-        return "CAST(DATEADD(s,$col,'1970-01-01') AS DATE)"            if $interval == 1;
-        return "CAST(DATEADD(s,$col/$interval,'1970-01-01') AS DATE)";
-    }
 }
 
 
 sub __stmt_epoch_to_timestamp {
     my ( $sf, $col, $interval ) = @_;
     #my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
-    my $dbms = $sf->{i}{dbms};
-    if ( $dbms =~ /^(?:Pg|DuckDB)\z/ ) { ##
+    my $driver = $sf->{i}{driver};
+    if ( $driver eq 'Pg' ) {
         return "TO_TIMESTAMP(${col}::bigint)::timestamptz"              if $interval == 1;
         return "TO_TIMESTAMP(${col}::bigint/$interval.0)::timestamptz";
     }
-    elsif ( $dbms eq 'Firebird' ) {
+    elsif ( $driver eq 'Firebird' ) {
         #my $firebird_major_version = $ax->major_server_version();
         my $firebird_major_version = 3; ##
         if ( $firebird_major_version >= 4 ) {
@@ -286,45 +279,33 @@ sub __stmt_epoch_to_timestamp {
             return "DATEADD(MILLISECOND,CAST($col AS BIGINT)/$interval,TIMESTAMP '1970-01-01')";
         }
     }
-    elsif ( $dbms eq 'Oracle' ) {
+    elsif ( $driver eq 'Oracle' ) {
         return "(TIMESTAMP '1970-01-01 00:00:00 UTC' + $col * INTERVAL '1' SECOND) AT TIME ZONE SESSIONTIMEZONE"            if $interval == 1;
         return "(TIMESTAMP '1970-01-01 00:00:00 UTC' + $col/$interval * INTERVAL '1' SECOND) AT TIME ZONE SESSIONTIMEZONE";
     }
-    elsif ( $dbms eq 'MSSQL' ) { # no timezone
-        return "DATEADD(s, $col,'1970-01-01 00:00:00')"                                                              if $interval == 1;
-        return "DATEADD(ms,$col%1000,DATEADD(s,$col/1000,CONVERT(datetime2(3),'1970-01-01 00:00:00.000')))"          if $interval == 1_000;
-        return "DATEADD(us,$col%1000000,DATEADD(s,$col/1000000,CONVERT(datetime2(6),'1970-01-01 00:00:00.000000')))" if $interval == 1_000_000;
-        #return "DATEADD(ns,$col%1000000000,DATEADD(s,$col/1000000000,CONVERT(datetime2(7),'1970-01-01 00:00:00.0000000')))"
-    }
 }
 
 
-
 sub __stmt_epoch_to_datetime {
     my ( $sf, $col, $interval ) = @_;
     #my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
-    my $dbms = $sf->{i}{dbms};
-    if ( $dbms eq 'SQLite' ) {
+    my $driver = $sf->{i}{driver};
+    if ( $driver eq 'SQLite' ) {
         return "DATETIME($col,'unixepoch','localtime')"                       if $interval == 1;
         return "DATETIME($col/$interval.0,'unixepoch','localtime','subsec')";
     }
-    elsif ( $dbms =~ /^(?:mysql|MariaDB)\z/ ) {   # DATE_FORMAT and STR_TO_DATE ##
+    elsif ( $driver =~ /^(?:mysql|MariaDB)\z/ ) {   # DATE_FORMAT and STR_TO_DATE ##
         # mysql: FROM_UNIXTIME doesn't work with negative timestamps
         return "FROM_UNIXTIME($col)"             if $interval == 1;
         return "FROM_UNIXTIME($col * 0.001)"     if $interval == 1_000;
         return "FROM_UNIXTIME($col * 0.000001)";
     }
-    elsif ( $dbms eq 'Pg' ) {
+    elsif ( $driver eq 'Pg' ) {
         return "TO_CHAR(TO_TIMESTAMP(${col}::bigint)::timestamp,'yyyy-mm-dd hh24:mi:ss')"                 if $interval == 1;
         return "TO_CHAR(TO_TIMESTAMP(${col}::bigint/$interval.0)::timestamp,'yyyy-mm-dd hh24:mi:ss.ff3')" if $interval == 1_000;
         return "TO_CHAR(TO_TIMESTAMP(${col}::bigint/$interval.0)::timestamp,'yyyy-mm-dd hh24:mi:ss.ff6')";
     }
-    elsif ( $dbms eq 'DuckDB' ) { # ###
-        return "TO_TIMESTAMP($col)::timestamp"                                            if $interval == 1;
-        return "STRFTIME(TO_TIMESTAMP($col/$interval)::timestamp,'%Y-%m-%d %H:%M:%S.%g')" if $interval == 1_000;
-        return "TO_TIMESTAMP($col/$interval)::timestamp";
-    }
-    elsif ( $dbms eq 'Firebird' ) {
+    elsif ( $driver eq 'Firebird' ) {
         #my $firebird_major_version = $ax->major_server_version();
         my $firebird_major_version = 3; ##
         if ( $firebird_major_version >= 4 ) {
@@ -340,16 +321,16 @@ sub __stmt_epoch_to_datetime {
             return "CAST(DATEADD(MILLISECOND,CAST($col AS BIGINT)/$interval.0,TIMESTAMP '1970-01-01') AS VARCHAR(24))";
         }
     }
-    elsif ( $dbms eq 'DB2' ) { # TO_DATE and TO_CHAR ##
+    elsif ( $driver eq 'DB2' ) { # TO_DATE and TO_CHAR ##
         return "TIMESTAMP('1970-01-01 00:00:00',0) + $col SECONDS"              if $interval == 1;
         return "TIMESTAMP('1970-01-01 00:00:00',3) + ($col/$interval) SECONDS"  if $interval == 1_000;
         return "TIMESTAMP('1970-01-01 00:00:00',6) + ($col/$interval) SECONDS";
     }
-    elsif ( $dbms eq 'Informix' ) { # TO_CHAR ##
+    elsif ( $driver eq 'Informix' ) { # TO_CHAR ##
         return "DBINFO('utc_to_datetime',$col)"            if $interval == 1;
         return "DBINFO('utc_to_datetime',$col/$interval)";
     }
-    elsif ( $dbms eq 'Oracle' ) {
+    elsif ( $driver eq 'Oracle' ) {
         return "TO_CHAR((TIMESTAMP '1970-01-01 00:00:00 UTC' + $col * INTERVAL '1' SECOND) AT TIME ZONE SESSIONTIMEZONE,'YYYY-MM-DD HH24:MI:SS')"                if $interval == 1;
         return "TO_CHAR((TIMESTAMP '1970-01-01 00:00:00 UTC' + $col/$interval * INTERVAL '1' SECOND) AT TIME ZONE SESSIONTIMEZONE,'YYYY-MM-DD HH24:MI:SS.FF3')"  if $interval == 1_000;
         return "TO_CHAR((TIMESTAMP '1970-01-01 00:00:00 UTC' + $col/$interval * INTERVAL '1' SECOND) AT TIME ZONE SESSIONTIMEZONE,'YYYY-MM-DD HH24:MI:SS.FF6')";
