@@ -22,28 +22,30 @@ sub new {
 
 
 sub __choose_a_column {
-    my ( $sf, $sql, $clause, $qt_cols, $info, $func ) = @_;
+    my ( $sf, $sql, $clause, $cols, $r_data ) = @_;
+    my $ext = App::DBBrowser::Table::Extensions->new( $sf->{i}, $sf->{o}, $sf->{d} );
+    my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
     my $tc = Term::Choose->new( $sf->{i}{tc_default} );
     my @pre = ( undef );
     if ( $sf->{i}{menu_addition} ) {
         push @pre, $sf->{i}{menu_addition};
     }
-    $info .= "\n" . $func . '(?)';
+    my $info = $ax->get_sql_info( $sql ) . $ext->nested_func_info( $r_data );
+
     while ( 1 ) {
         # Choose
         my $choice = $tc->choose(
-            [ @pre, @$qt_cols ],
+            [ @pre, @$cols ],
             { %{$sf->{i}{lyt_h}}, info => $info, prompt => 'Column:' }
         );
         if ( ! defined $choice ) {
             return;
         }
         elsif ( $choice eq $sf->{i}{menu_addition} ) {
-            my $ext = App::DBBrowser::Table::Extensions->new( $sf->{i}, $sf->{o}, $sf->{d} );
             # from 'window_function': to avoid window function in window function
             my $complex_col = $ext->column(
-                $sql, $clause, {},
-                { from =>'window_function', info => $info }
+                $sql, $clause, $r_data,
+                { from =>'window_function' }
             );
             if ( ! defined $complex_col ) {
                 next;
@@ -58,20 +60,35 @@ sub __choose_a_column {
 sub __get_win_func_stmt {
     my ( $sf, $win_func_data ) = @_;
     my $win_func_stmt = $win_func_data->{func};
-    $win_func_stmt .= sprintf '(%s)', $win_func_data->{args} // '';
+    $win_func_stmt .= sprintf '(%s)', $win_func_data->{args_str} // '';
     my @win_definition;
-    for my $stmt ( qw(partition_by_stmt order_by_stmt frame_clause) ) {
+    for my $stmt ( qw(partition_by_stmt order_by_stmt) ) {
         if ( length $win_func_data->{$stmt} ) {
             push @win_definition, $win_func_data->{$stmt};
         }
     }
+    if ( length $win_func_data->{frame_mode} ) {
+        push @win_definition, $win_func_data->{frame_mode};
+        if ( length $win_func_data->{frame_start} && length $win_func_data->{frame_end} ) {
+            push @win_definition,  "BETWEEN " . $win_func_data->{frame_start} . " AND " . $win_func_data->{frame_end};
+        }
+        elsif ( length $win_func_data->{frame_start} ) {
+            push @win_definition, $win_func_data->{frame_start};
+        }
+        elsif ( length $win_func_data->{frame_end} ) {
+            push @win_definition, $win_func_data->{frame_end};
+        }
+        if ( length $win_func_data->{frame_exclusion} ) {
+            push @win_definition, $win_func_data->{frame_exclusion};
+        }
+    }
     $win_func_stmt .= sprintf ' OVER (%s)', join( ' ',  @win_definition ) // '';
     return $win_func_stmt;
 }
 
 
 sub window_function {
-    my ( $sf, $sql, $clause, $qt_cols, $opt ) = @_;
+    my ( $sf, $sql, $clause, $cols, $r_data ) = @_;
     my $tc = Term::Choose->new( $sf->{i}{tc_default} );
     my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
     my $ext = App::DBBrowser::Table::Extensions->new( $sf->{i}, $sf->{o}, $sf->{d} );
@@ -95,12 +112,15 @@ sub window_function {
     my @func_with_offset_and_default = ( 'LAG', 'LEAD' );
     my $rx_func_with_offset_and_default = join( '|', map { quotemeta } @func_with_offset_and_default );
 
-    my $info = $opt->{info} // $ax->get_sql_info( $sql );
-    my $win_func_data = {};
+    my $info_sql = $ax->get_sql_info( $sql );
+    push @$r_data, [ 'win' ];
     my $hidden = 'Window function:';
     my $old_idx_wf = 1;
 
     WINDOW_FUNCTION: while( 1 ) {
+        my $win_func_data = {};
+        $r_data->[-1] = [ 'win' ];
+        my $info = $info_sql . $ext->nested_func_info( $r_data );
         my @pre = ( $hidden, undef );
         my $menu = [ @pre, map { '- ' . $_ } @functions ];
         # Choose
@@ -110,6 +130,7 @@ sub window_function {
               default => $old_idx_wf, undef => '<=' }
         );
         if ( ! defined $idx_wf || ! defined $menu->[$idx_wf] ) {
+            pop @$r_data;
             return;
         }
         if ( $sf->{o}{G}{menu_memory} ) {
@@ -125,78 +146,76 @@ sub window_function {
         }
         my $func = $functions[$idx_wf-@pre];
         $win_func_data->{func} = $func;
+        $r_data->[-1] = [ 'win', $sf->__get_win_func_stmt( $win_func_data ) ];
 
         COLUMN: while ( 1 ) {
-            if ( exists $win_func_data->{args} ) {
-                delete $win_func_data->{args};
-            }
-            my $tmp_info = $info . "\n" . $sf->__get_win_func_stmt( $win_func_data );
-            my $qt_col;
+            $win_func_data->{args_str} = '';
+            my $col;
             if ( $func =~ /^$rx_func_count_all\z/i ) {
-                $qt_col = '*';
+                $col = '*';
                 $win_func_data->{func} = $func =~ s/\*\z//r;
             }
             elsif ( $func =~ /^(?:$rx_func_no_col)\z/i ) {
-                $qt_col = '';
+                $col = '';
             }
             elsif ( $func =~ /^(?:$rx_func_col_is_number)\z/i ) {
                 # Readline
-                $qt_col = $ext->argument( $sql, $clause, { info => $info . "\n" . $func . '(n)', history => undef, prompt => 'n = ', is_numeric => 1 } );
-                if ( ! length $qt_col || $qt_col eq "''" ) {
+                $col = $ext->argument(
+                    $sql, $clause, $r_data,
+                    { history => undef, prompt => 'n = ', is_numeric => 1 }
+                );
+                if ( ! length $col || $col eq "''" ) {
                     next WINDOW_FUNCTION;
                 }
             }
             else {
-                $qt_col = $sf->__choose_a_column( $sql, $clause, $qt_cols, $info, $func );
-                if ( ! defined $qt_col ) {
-                    delete $win_func_data->{func};
+                $col = $sf->__choose_a_column( $sql, $clause, $cols, $r_data );
+                if ( ! defined $col ) {
                     next WINDOW_FUNCTION;
                 }
             }
-            my $args = $qt_col;
-            $win_func_data->{args} = $args;
+            $win_func_data->{args_str} = $col;
+            $r_data->[-1] = [ 'win', $sf->__get_win_func_stmt( $win_func_data ) ];
             if ( $func =~ /^(?:$rx_func_with_offset)\z/i ) {
-                $tmp_info = $info . "\n" . $sf->__get_win_func_stmt( $win_func_data );
                 # Readline
-                my $offset = $ext->argument( $sql, $clause, { info => $tmp_info, history => undef, prompt => 'offset: ', is_numeric => 1 } );
-                #if ( ! defined $offset ) {
-                #    next WINDOW_FUNCTION;
-                #}
+                my $offset = $ext->argument(
+                    $sql, $clause, $r_data,
+                    { history => undef, prompt => 'offset: ', is_numeric => 1 }
+                );
                 if ( length $offset && $offset ne "''" ) {
-                    $args .= ',' . $offset;
-                    $win_func_data->{args} = $args;
-                    $tmp_info = $info . "\n" . $sf->__get_win_func_stmt( $win_func_data );
+                    $win_func_data->{args_str} .= ',' . $offset;
+                    $r_data->[-1] = [ 'win', $sf->__get_win_func_stmt( $win_func_data ) ];
                     if ( $func =~ /^(?:$rx_func_with_offset_and_default)\z/i ) {
-                        my $is_numeric = $ax->is_numeric_datatype( $sql, $qt_col );
+                        my $is_numeric = $ax->is_numeric_datatype( $sql, $col );
                         # Readline
-                        my $default_value = $ext->argument( $sql, $clause, { info => $tmp_info, history => undef, prompt => 'default: ', is_numeric => $is_numeric } );
-                        #if ( ! defined $default_value) {
-                        #    next WINDOW_FUNCTION;
-                        #}
+                        my $default_value = $ext->argument(
+                            $sql, $clause, $r_data,
+                            { history => undef, prompt => 'default: ', is_numeric => $is_numeric }
+                        );
                         if ( length $default_value && $default_value ne "''" ) {
-                            $args .= ',' . $default_value;
-                            $win_func_data->{args} = $args;
+                            $win_func_data->{args_str} .= ',' . $default_value;
+                            $r_data->[-1] = [ 'win', $sf->__get_win_func_stmt( $win_func_data ) ];
                         }
                     }
                 }
             }
-            my @bu;
             my $old_idx = 0;
 
             WINDOW_DEFINITION: while( 1 ) {
                 my ( $partition_by, $order_by, $frame_clause ) = ( '- Partition by', '- Order by', '- Frame clause' );
                 my @pre = ( undef, $sf->{i}{confirm} );
                 my $menu = [ @pre, $partition_by, $order_by, $frame_clause ];
-                my $tmp_info = $info . "\n" . $sf->__get_win_func_stmt( $win_func_data );
+                my $info = $info_sql . $ext->nested_func_info( $r_data );
                 # Choose
                 my $idx_wd = $tc->choose(
                     $menu,
-                    { %{$sf->{i}{lyt_v}}, info => $tmp_info, prompt => '', index => 1, default => $old_idx,
+                    { %{$sf->{i}{lyt_v}}, info => $info, prompt => '', index => 1, default => $old_idx,
                       undef => $sf->{i}{back} }
                 );
                 if ( ! defined $idx_wd || ! defined $menu->[$idx_wd] ) {
-                    if ( @bu ) {
-                        $win_func_data = pop @bu;
+                    if ( $win_func_data->{partition_by_stmt} || $win_func_data->{order_by_stmt} || $win_func_data->{frame_mode} ) {
+                        $win_func_data = { func => $win_func_data->{func}, args_str => $win_func_data->{args_str} };
+                        $r_data->[-1] = [ 'win', $sf->__get_win_func_stmt( $win_func_data ) ];
                         next WINDOW_DEFINITION;
                     }
                     if ( $func =~ /^(?:$rx_func_count_all|$rx_func_no_col)\z/ ) {
@@ -211,29 +230,20 @@ sub window_function {
                     }
                     $old_idx = $idx_wd;
                 }
-                push @bu, { %$win_func_data };
                 my $wd = $menu->[$idx_wd];
                 if ( $wd eq $sf->{i}{confirm} ) {
+                    pop @$r_data;
                     my $win_func_stmt = $sf->__get_win_func_stmt( $win_func_data );
                     return $win_func_stmt;
                 }
                 elsif ( $wd eq $partition_by ) {
-                    my $ret = $sf->__add_partition_by( $sql, $clause, $qt_cols, $win_func_data );
-                    if ( ! $ret ) {
-                        pop @bu;
-                    }
+                    $sf->__add_partition_by( $sql, $clause, $cols, $r_data, $win_func_data );
                 }
                 elsif ( $wd eq $order_by ) {
-                    my $ret = $sf->__add_order_by( $sql, $clause, $qt_cols, $win_func_data );
-                    if ( ! $ret ) {
-                        pop @bu;
-                    }
+                    $sf->__add_order_by( $sql, $clause, $cols, $r_data, $win_func_data );
                 }
                 elsif ( $wd eq $frame_clause ) {
-                    my $ret = $sf->__add_frame_clause( $sql, $clause, $win_func_data );
-                    if ( ! $ret ) {
-                        pop @bu;
-                    }
+                    $sf->__add_frame_clause( $sql, $clause, $r_data, $win_func_data );
                 }
             }
         }
@@ -242,148 +252,163 @@ sub window_function {
 
 
 sub __add_partition_by {
-    my ( $sf, $sql, $clause, $qt_cols, $win_func_data ) = @_;
+    my ( $sf, $sql, $clause, $cols, $r_data, $win_func_data ) = @_;
     my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
+    my $ext = App::DBBrowser::Table::Extensions->new( $sf->{i}, $sf->{o}, $sf->{d} );
     my $tc = Term::Choose->new( $sf->{i}{tc_default} );
-    my @partition_by_cols;
+    $win_func_data->{partition_by_cols} //= [];
     my @pre = ( undef, $sf->{i}{ok} );
     if ( $sf->{o}{enable}{extended_cols} ) {
         push @pre, $sf->{i}{menu_addition};
     }
-    my $menu = [ @pre, @$qt_cols ];
-    my $info = $ax->get_sql_info( $sql );
+    my $menu = [ @pre, @$cols ];
+    my $info_sql = $ax->get_sql_info( $sql );
 
     PARTITION_BY: while ( 1 ) {
-        my $partition_by_stmt = "PARTITION BY " . join ',', @partition_by_cols;
-        my $tmp_info = $info . "\n" . $sf->__get_win_func_stmt( $win_func_data ) . "\n" . $partition_by_stmt;
+        if ( @{$win_func_data->{partition_by_cols}} ) {
+            $win_func_data->{partition_by_stmt} = "PARTITION BY " . join ',', @{$win_func_data->{partition_by_cols}};
+        }
+        else {
+            $win_func_data->{partition_by_stmt} = "PARTITION BY";
+        }
+        $r_data->[-1] = [ 'win', $sf->__get_win_func_stmt( $win_func_data ) ];
+        my $info = $info_sql . $ext->nested_func_info( $r_data );
         # Choose
         my @idx = $tc->choose(
             $menu,
-            { %{$sf->{i}{lyt_h}}, info => $tmp_info, meta_items => [ 0 .. $#pre - 1 ], no_spacebar => [ $#pre ],
+            { %{$sf->{i}{lyt_h}}, info => $info, meta_items => [ 0 .. $#pre - 1 ], no_spacebar => [ $#pre ],
               include_highlighted => 2, index => 1, prompt => 'Columns:' }
         );
-        $ax->print_sql_info( $tmp_info );
+        $ax->print_sql_info( $info );
         if ( ! $idx[0] ) {
-            if ( @partition_by_cols ) {
-                pop @partition_by_cols;
+            if ( @{$win_func_data->{partition_by_cols}} ) {
+                pop @{$win_func_data->{partition_by_cols}};
                 next PARTITION_BY;
             }
+            delete $win_func_data->{partition_by_stmt};
+            $r_data->[-1] = [ 'win', $sf->__get_win_func_stmt( $win_func_data ) ];
             return;
         }
         elsif ( $menu->[$idx[0]] eq $sf->{i}{ok} ) {
             shift @idx;
-            push @partition_by_cols, @{$menu}[@idx];
-            if ( ! @partition_by_cols ) {
+            push @{$win_func_data->{partition_by_cols}}, @{$menu}[@idx];
+            if ( ! @{$win_func_data->{partition_by_cols}} ) {
                 delete $win_func_data->{partition_by_stmt};
             }
             else {
-                $win_func_data->{partition_by_stmt} = "PARTITION BY " . join ',', @partition_by_cols;
+                $win_func_data->{partition_by_stmt} = "PARTITION BY " . join ',', @{$win_func_data->{partition_by_cols}};
             }
+            $r_data->[-1] = [ 'win', $sf->__get_win_func_stmt( $win_func_data ) ];
             return 1;
         }
         elsif ( $menu->[$idx[0]] eq $sf->{i}{menu_addition} ) {
             my $ext = App::DBBrowser::Table::Extensions->new( $sf->{i}, $sf->{o}, $sf->{d} );
             my $complex_column = $ext->column(
-                $sql, $clause, {},
-                { info => $tmp_info, from => 'window_function' }
+                $sql, $clause, $r_data,
+                { from => 'window_function' }
             );
             if ( defined $complex_column ) {
-                push @partition_by_cols, $complex_column;
+                push @{$win_func_data->{partition_by_cols}}, $complex_column;
             }
             next PARTITION_BY;
         }
-        push @partition_by_cols, @{$menu}[@idx];
+        push @{$win_func_data->{partition_by_cols}}, @{$menu}[@idx];
     }
 }
 
 
 sub __add_order_by {
-    my ( $sf, $sql, $clause, $qt_cols, $win_func_data ) = @_;
+    my ( $sf, $sql, $clause, $cols, $r_data, $win_func_data ) = @_;
     my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
+    my $ext = App::DBBrowser::Table::Extensions->new( $sf->{i}, $sf->{o}, $sf->{d} );
     my $tc = Term::Choose->new( $sf->{i}{tc_default} );
+    $win_func_data->{order_by_cols} //= [];
     my @pre = ( undef, $sf->{i}{ok} );
     if ( $sf->{o}{enable}{extended_cols} ) {
         push @pre, $sf->{i}{menu_addition};
     }
-    my $info = $ax->get_sql_info( $sql );
-    my $col_sep = ' ';
-    my $order_by_stmt =  "ORDER BY";
-    my @bu;
+    my $info_sql = $ax->get_sql_info( $sql );
 
     ORDER_BY: while ( 1 ) {
-        my $tmp_info = $info . "\n" . $sf->__get_win_func_stmt( $win_func_data ) . "\n" . $order_by_stmt;
+        if ( @{$win_func_data->{order_by_cols}} ) {
+            $win_func_data->{order_by_stmt} = "ORDER BY " . join ',', @{$win_func_data->{order_by_cols}};
+        }
+        else {
+            $win_func_data->{order_by_stmt} = "ORDER BY";
+        }
+        $r_data->[-1] = [ 'win', $sf->__get_win_func_stmt( $win_func_data ) ];
+        my $info = $info_sql . $ext->nested_func_info( $r_data );
         # Choose
         my $col = $tc->choose(
-            [ @pre, @$qt_cols ],
-            { %{$sf->{i}{lyt_h}}, info => $tmp_info, prompt => 'Column:' }
+            [ @pre, @$cols ],
+            { %{$sf->{i}{lyt_h}}, info => $info, prompt => 'Column:' }
         );
-        $ax->print_sql_info( $tmp_info );
+        $ax->print_sql_info( $info );
         if ( ! defined $col ) {
-            if ( @bu ) {
-                ( $order_by_stmt, $col_sep ) = @{pop @bu};
+            if ( @{$win_func_data->{order_by_cols}} ) {
+                pop @{$win_func_data->{order_by_cols}};
                 next ORDER_BY;
             }
             delete $win_func_data->{order_by_stmt};
+            $r_data->[-1] = [ 'win', $sf->__get_win_func_stmt( $win_func_data ) ];
             return
         }
         if ( $col eq $sf->{i}{ok} ) {
-            if ( $col_sep eq ' ' ) {
+            if ( ! @{$win_func_data->{order_by_cols}} ) {
                 delete $win_func_data->{order_by_stmt};
             }
             else {
-                $win_func_data->{order_by_stmt} = $order_by_stmt;
+                $win_func_data->{order_by_stmt} = "ORDER BY " . join ',', @{$win_func_data->{order_by_cols}};
             }
+            $r_data->[-1] = [ 'win', $sf->__get_win_func_stmt( $win_func_data ) ];
             return 1;
         }
         elsif ( $col eq $sf->{i}{menu_addition} ) {
             my $ext = App::DBBrowser::Table::Extensions->new( $sf->{i}, $sf->{o}, $sf->{d} );
             my $complex_column = $ext->column(
-                $sql, $clause, {},
-                { info => $tmp_info, from => 'window_function' }
+                $sql, $clause, $r_data,
+                { from => 'window_function' }
             );
             if ( ! defined $complex_column ) {
-                if ( @bu ) {
-                    ( $order_by_stmt, $col_sep ) = @{pop @bu};
-                }
                 next ORDER_BY;
             }
             $col = $complex_column;
         }
-        push @bu, [ $order_by_stmt, $col_sep ];
-        $order_by_stmt .= $col_sep . $col;
-        $tmp_info = $info . "\n" . $sf->__get_win_func_stmt( $win_func_data ) . "\n" . $order_by_stmt;
+        push @{$win_func_data->{order_by_cols}}, $col;
+        $win_func_data->{order_by_stmt} = "ORDER BY " . join ',', @{$win_func_data->{order_by_cols}};
+        $r_data->[-1] = [ 'win', $sf->__get_win_func_stmt( $win_func_data ) ];
+        $info = $info_sql . $ext->nested_func_info( $r_data );
         # Choose
         my $direction = $tc->choose(
             [ undef, "ASC", "DESC" ],
-            { %{$sf->{i}{lyt_h}}, info => $tmp_info, prompt => '' }
+            { %{$sf->{i}{lyt_h}}, info => $info }
         );
-        $ax->print_sql_info( $tmp_info );
+        $ax->print_sql_info( $info );
         if ( ! defined $direction ){
-            ( $order_by_stmt, $col_sep ) = @{pop @bu};
+            pop @{$win_func_data->{order_by_cols}};
             next ORDER_BY;
         }
         else {
-            $order_by_stmt .= ' ' . $direction;
+            $win_func_data->{order_by_cols}[-1] .= ' ' . $direction;
         }
-        $col_sep = ', ';
     }
 }
 
 
 sub __add_frame_clause {
-    my ( $sf, $sql, $clause, $win_func_data ) = @_;
+    my ( $sf, $sql, $clause, $r_data, $win_func_data ) = @_;
     my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
+    my $ext = App::DBBrowser::Table::Extensions->new( $sf->{i}, $sf->{o}, $sf->{d} );
     my $tc = Term::Choose->new( $sf->{i}{tc_default} );
     my @frame_clause_modes = ( 'ROWS', 'RANGE' );
     if ( $sf->{i}{driver} =~ /^(?:SQLite|Pg|Oracle)\z/ ) {
         push @frame_clause_modes, 'GROUPS';
     }
-    my $info = $ax->get_sql_info( $sql );
-    my $win_func_stmt = $sf->__get_win_func_stmt( $win_func_data );
-    $info .= "\n" . $win_func_stmt;
+    my $info_sql = $ax->get_sql_info( $sql );
     my $old_idx_fc = 0;
 
      FRAME_CLAUSE: while ( 1 ) {
+        my $info = $info_sql . $ext->nested_func_info( $r_data );
         my @pre = ( undef );
         my $menu = [ @pre, map { '- ' . $_ } @frame_clause_modes ];
         # Choose
@@ -403,8 +428,8 @@ sub __add_frame_clause {
             $old_idx_fc = $idx_fc;
         }
         my $frame_mode = $frame_clause_modes[$idx_fc-@pre];
-        my $frame_clause_data = { frame_mode => $frame_mode };
-        my @bu;
+        $win_func_data->{frame_mode} = $frame_mode;
+        $r_data->[-1] = [ 'win', $sf->__get_win_func_stmt( $win_func_data ) ];
         my $old_idx_fe = 0;
 
         FRAME_END_AND_EXCLUSION: while ( 1 ) {
@@ -415,18 +440,21 @@ sub __add_frame_clause {
             if ( $sf->{i}{driver} =~ /^(?:SQLite|Pg|Oracle)\z/ ) {
                 push @$menu, $frame_exclusion;
             }
-            my $tmp_info = $info . "\n" . $sf->__get_frame_clause_stmt( $frame_clause_data );
+            my $info = $info_sql . "\n" . $sf->__get_win_func_stmt( $win_func_data );
             # Choose
             my $idx_fe = $tc->choose(
                 $menu,
-                { %{$sf->{i}{lyt_v}}, info => $tmp_info, index => 1, default => $old_idx_fe, prompt => 'Frame clause:', undef => 'Back' }
+                { %{$sf->{i}{lyt_v}}, info => $info, index => 1, default => $old_idx_fe, prompt => 'Frame clause:', undef => 'Back' }
             );
-            $ax->print_sql_info( $tmp_info );
+            $ax->print_sql_info( $info );
             if ( ! defined $idx_fe || ! defined $menu->[$idx_fe] ) {
-                if ( @bu ) {
-                    $frame_clause_data = pop @bu;
+                if ( $win_func_data->{frame_start} || $win_func_data->{frame_end} || $win_func_data->{frame_exclusion} ) {
+                    delete @{$win_func_data}{qw(frame_start frame_end frame_exclusion)};
+                    $r_data->[-1] = [ 'win', $sf->__get_win_func_stmt( $win_func_data ) ];
                     next FRAME_END_AND_EXCLUSION;
                 }
+                delete $win_func_data->{frame_mode};
+                $r_data->[-1] = [ 'win', $sf->__get_win_func_stmt( $win_func_data ) ];
                 next FRAME_CLAUSE;
             }
             if ( $sf->{o}{G}{menu_memory} ) {
@@ -438,30 +466,16 @@ sub __add_frame_clause {
             }
             my $choice = $menu->[$idx_fe];
             if ( $choice eq $confirm ) {
-                $win_func_data->{frame_clause} = $sf->__get_frame_clause_stmt( $frame_clause_data );
                 return 1;
             }
-            push @bu, { %$frame_clause_data };
             if ( $choice eq $frame_start ) {
-                my $ret = $sf->__add_frame_start_or_end( $sql, $clause, $frame_clause_data, $info, 'frame_start' );
-                if ( ! defined $ret ) {
-                    pop @bu;
-                    next FRAME_END_AND_EXCLUSION;
-                }
+                $sf->__add_frame_start_or_end( $sql, $clause, $r_data, $win_func_data, 'frame_start' );
             }
             elsif ( $choice eq $frame_end ) {
-                my $ret = $sf->__add_frame_start_or_end( $sql, $clause, $frame_clause_data, $info, 'frame_end' );
-                if ( ! defined $ret ) {
-                    pop @bu;
-                    next FRAME_END_AND_EXCLUSION;
-                }
+                $sf->__add_frame_start_or_end( $sql, $clause, $r_data, $win_func_data, 'frame_end' );
             }
             else {
-                my $ret = $sf->__add_frame_exclusion( $frame_clause_data, $info );
-                if ( ! defined $ret ) {
-                    pop @bu;
-                    next FRAME_END_AND_EXCLUSION;
-                }
+                $sf->__add_frame_exclusion( $sql, $r_data, $win_func_data );
             }
         }
     }
@@ -469,8 +483,9 @@ sub __add_frame_clause {
 
 
 sub __add_frame_start_or_end {
-    my ( $sf, $sql, $clause, $frame_clause_data, $info, $pos ) = @_;
+    my ( $sf, $sql, $clause, $r_data, $win_func_data, $pos ) = @_;
     my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
+    my $ext = App::DBBrowser::Table::Extensions->new( $sf->{i}, $sf->{o}, $sf->{d} );
     my $tc = Term::Choose->new( $sf->{i}{tc_default} );
     my ( @frame_point_types, $prompt );
     if ( $pos eq 'frame_start' ) {
@@ -481,94 +496,87 @@ sub __add_frame_start_or_end {
         @frame_point_types = ( 'n PRECEDING', 'CURRENT ROW', 'n FOLLOWING', 'UNBOUNDED FOLLOWING' );
         $prompt = 'Frame end:';
     }
-    my $reset = '  Reset';
-    my @pre = ( undef );
+    my $confirm = '-OK-';
+    my @pre = ( undef, $confirm );
+    my $info_sql = $ax->get_sql_info( $sql );
 
     FRAME_START: while ( 1 ) {
-        my $tmp_info = $info . "\n" . $sf->__get_frame_clause_stmt( $frame_clause_data );
-        my $menu = [ @pre, map( '- ' . $_, @frame_point_types ), $reset ];
+        my $info = $info_sql . $ext->nested_func_info( $r_data );
+        my $menu = [ @pre, map( '- ' . $_, @frame_point_types ) ];
         # Choose
         my $point = $tc->choose(
             $menu,
-            { %{$sf->{i}{lyt_v}}, info => $tmp_info, prompt => $prompt, undef => '<=' }
+            { %{$sf->{i}{lyt_v}}, info => $info, prompt => $prompt, undef => '<=' }
         );
-        $ax->print_sql_info( $tmp_info );
+        $ax->print_sql_info( $info );
         if ( ! defined $point ) {
+            delete $win_func_data->{$pos};
+            $r_data->[-1] = [ 'win', $sf->__get_win_func_stmt( $win_func_data ) ];
             return;
         }
-        elsif ( $point eq $reset ) {
-            delete $frame_clause_data->{$pos};
+        elsif ( $point eq $confirm ) {
+            return 1;
         }
         else {
             my $ext = App::DBBrowser::Table::Extensions->new( $sf->{i}, $sf->{o}, $sf->{d} );
             $point =~ s/-\s//;
-            $frame_clause_data->{$pos} = $point;
+            $win_func_data->{$pos} = $point;
+            $r_data->[-1] = [ 'win', $sf->__get_win_func_stmt( $win_func_data ) ];
             if ( $point =~ /^n / ) {
-                my $tmp_info = $info . "\n" . $sf->__get_frame_clause_stmt( $frame_clause_data );
-                my $offset = $ext->argument( $sql, $clause, { info => $tmp_info, history => undef, prompt => 'n = ', is_numeric => 1 } );
+                my $offset = $ext->argument(
+                    $sql, $clause, $r_data,
+                    { history => undef, prompt => 'n = ', is_numeric => 1 }
+                );
                 if ( ! length $offset || $offset eq "''" ) {
+                    delete $win_func_data->{$pos};
+                    $r_data->[-1] = [ 'win', $sf->__get_win_func_stmt( $win_func_data ) ];
                     next FRAME_START;
                 }
                 $point =~ s/^n/$offset/;
-                $frame_clause_data->{$pos} = $point;
+                $win_func_data->{$pos} = $point;
+                $r_data->[-1] = [ 'win', $sf->__get_win_func_stmt( $win_func_data ) ];
             }
         }
-        return 1;
     }
 }
 
 
 sub __add_frame_exclusion {
-    my ( $sf, $frame_clause_data, $info ) = @_;
+    my ( $sf, $sql, $r_data, $win_func_data ) = @_;
     my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
+    my $ext = App::DBBrowser::Table::Extensions->new( $sf->{i}, $sf->{o}, $sf->{d} );
     my $tc = Term::Choose->new( $sf->{i}{tc_default} );
     my @exclusion_types = ( 'EXCLUDE CURRENT ROW', 'EXCLUDE GROUP', 'EXCLUDE TIES', 'EXCLUDE NO OTHERS' );
-    my $reset = '  Reset';
-    my @pre = ( undef );
-    my $menu = [ @pre, map( '- ' . $_, @exclusion_types ), $reset ];
-    my $tmp_info = $info . "\n" . $sf->__get_frame_clause_stmt( $frame_clause_data );
-    # Choose
-    my $frame_exclusion = $tc->choose(
-        $menu,
-        { %{$sf->{i}{lyt_v}}, info => $tmp_info, prompt => 'Frame exclusion:', undef => '<=' }
-    );
-    $ax->print_sql_info( $tmp_info );
-    if ( ! defined $frame_exclusion ) {
-        return;
+    my $confirm = '-OK-';
+    my @pre = ( undef, $confirm );
+    my $menu = [ @pre, map( '- ' . $_, @exclusion_types ) ];
+
+    FRAME_EXCLUSION: while ( 1 ) {
+        my $info = $ax->get_sql_info( $sql ) . $ext->nested_func_info( $r_data );
+        # Choose
+        my $frame_exclusion = $tc->choose(
+            $menu,
+            { %{$sf->{i}{lyt_v}}, info => $info, prompt => 'Frame exclusion:', undef => '<=' }
+        );
+        $ax->print_sql_info( $info );
+        if ( ! defined $frame_exclusion ) {
+            delete $win_func_data->{frame_exclusion};
+            $r_data->[-1] = [ 'win', $sf->__get_win_func_stmt( $win_func_data ) ];
+            return;
+        }
+        elsif ( $frame_exclusion eq $confirm ) {
+            return 1;
+        }
+        else {
+            $frame_exclusion =~ s/^-\s//;
+            $win_func_data->{frame_exclusion} = $frame_exclusion;
+            $r_data->[-1] = [ 'win', $sf->__get_win_func_stmt( $win_func_data ) ];
+        }
     }
-    elsif ( $frame_exclusion eq $reset ) {
-        delete $frame_clause_data->{frame_exclusion};
-    }
-    else {
-        $frame_exclusion =~ s/^-\s//;
-        $frame_clause_data->{frame_exclusion} = $frame_exclusion;
-    }
-    return 1;
 }
 
 
-sub __get_frame_clause_stmt {
-    my ( $sf, $frame_clause_data ) = @_;
-    my $frame_clause_stmt = $frame_clause_data->{frame_mode};
-    if ( length $frame_clause_data->{frame_start} && length $frame_clause_data->{frame_end} ) {
-        $frame_clause_stmt .= " BETWEEN " . $frame_clause_data->{frame_start} . " AND " . $frame_clause_data->{frame_end};
-    }
-    elsif ( length $frame_clause_data->{frame_start} ) {
-        $frame_clause_stmt .= " " . $frame_clause_data->{frame_start};
-    }
-    elsif ( length $frame_clause_data->{frame_end} ) {
-         $frame_clause_stmt .= " " . $frame_clause_data->{frame_end};
-    }
-    if ( length $frame_clause_data->{frame_exclusion} ) {
-        $frame_clause_stmt .= " " . $frame_clause_data->{frame_exclusion};
-    }
-    return $frame_clause_stmt;
-}
-
-
-
 
 1;
 
-
 __END__
