@@ -26,15 +26,12 @@ sub new {
 
 
 sub add_operator_and_value {
-    my ( $sf, $sql, $clause, $stmt, $qt_col ) = @_;
+    my ( $sf, $sql, $clause, $stmt, $col, $r_data ) = @_;
     my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
     my $tc = Term::Choose->new( $sf->{i}{tc_default} );
     my @operators = @{$sf->{o}{G}{operators}};
     my $not_equal = ( any { $_ =~ /^\s?!=\s?\z/ } @operators ) ? "!=" : "<>";
-    if ( $clause eq 'on' ) {
-        @operators = ( " = ", " $not_equal ", " < ", " > ", " >= ", " <= ", "LIKE", "NOT LIKE" );
-    }
-    elsif ( ! length $qt_col ) {
+    if ( ! length $col ) {
         $sql->{$stmt} =~ s/\s\z//;
         @operators = ( "EXISTS", "NOT EXISTS" );
     }
@@ -59,7 +56,7 @@ sub add_operator_and_value {
         #}
         #else {
             my @pre = ( undef );
-            my $info = $ax->get_sql_info( $sql );
+            my $info = $sf->info_add_condition( $sql, $clause, $stmt, $r_data );
             # Choose
             $operator = $tc->choose(
                 [ @pre, @operators ],
@@ -72,16 +69,15 @@ sub add_operator_and_value {
             }
         #}
         $operator =~ s/^\s+|\s+\z//g;
-        $ax->print_sql_info( $ax->get_sql_info( $sql ) );
         if ( $operator =~ /(?:REGEXP(?:_i)?|SIMILAR\sTO)\z/ ) {
             my $not_match = $operator =~ /^NOT/ ? 1 : 0;
             my $case_sensitive = $operator =~ /REGEXP_i\z/ ? 0 : 1;
-            my $regex_op = $sf->pattern_match( $qt_col, $not_match, $case_sensitive );
+            my $regex_op = $sf->__pattern_match( $col, $not_match, $case_sensitive );
             if ( ! $regex_op ) {
                 next OPERATOR if @operators > 1;
                 return;
             }
-            $sql->{$stmt} =~ s/ (?: (?<=\() | \s ) \Q$qt_col\E \z //x;
+            $sql->{$stmt} =~ s/ (?: (?<=\() | \s ) \Q$col\E \z //x;
             if ( $sql->{$stmt} =~ /\(\z/ ) {
                 $regex_op =~ s/^\s//;
             }
@@ -90,7 +86,7 @@ sub add_operator_and_value {
         elsif ( $operator =~ /^(?:ALL|ANY)\z/) {
             my @comb_op = ( "= $operator", "$not_equal $operator", "> $operator", "< $operator", ">= $operator", "<= $operator" );
             my @pre = ( undef );
-            my $info = $ax->get_sql_info( $sql );
+            my $info = $sf->info_add_condition( $sql, $clause, $stmt, $r_data );
             # Choose
             $operator = $tc->choose(
                 [ @pre, @comb_op ],
@@ -114,14 +110,7 @@ sub add_operator_and_value {
         else {
             $sql->{$stmt} .= ' ' . $operator;
         }
-        $ax->print_sql_info( $ax->get_sql_info( $sql ) );
-        my $ok;
-        if ( $clause eq 'on' ) {
-            $ok = $sf->choose_and_add_column( $sql, $clause, $stmt );
-        }
-        else {
-            $ok = $sf->read_and_add_value( $sql, $clause, $stmt, $qt_col, $operator );
-        }
+        my $ok = $sf->read_and_add_value( $sql, $clause, $stmt, $col, $operator, $r_data );
         if ( $ok ) {
             return 1;
         }
@@ -134,47 +123,29 @@ sub add_operator_and_value {
 }
 
 
-sub choose_and_add_column {
-    my ( $sf, $sql, $clause, $stmt ) = @_;
+sub info_add_condition {
+    my ( $sf, $sql, $clause, $stmt, $r_data ) = @_;
     my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
-    my $tc = Term::Choose->new( $sf->{i}{tc_default} );
-    my @pre = ( undef );
-    if ( $sf->{o}{enable}{extended_cols} ) {
-        push @pre, $sf->{i}{menu_addition};
+    my $info;
+    if ( @{$r_data//[]} ) {
+        $info = $ax->get_sql_info( $sql );
+        my $ext = App::DBBrowser::Table::Extensions->new( $sf->{i}, $sf->{o}, $sf->{d} );
+        $r_data->[-1][-1] = $sql->{$stmt};
+        $info .= $ext->nested_func_info( $r_data );
     }
-    my @choices = @{$sql->{cols_join_condition}};
-
-    COL: while ( 1 ) {
-        my $info = $ax->get_sql_info( $sql );
-        # Choose
-        my $qt_col = $tc->choose(
-            [ @pre, @choices ],
-            { %{$sf->{i}{lyt_h}}, info => $info }
-        );
-        $ax->print_sql_info( $info );
-        if ( ! defined $qt_col ) {
-            return;
-        }
-        if ( $qt_col eq $sf->{i}{menu_addition} ) {
-            my $ext = App::DBBrowser::Table::Extensions->new( $sf->{i}, $sf->{o}, $sf->{d} );
-            my $complex_col = $ext->column( $sql, $clause );
-            if ( ! defined $complex_col ) {
-                next COL;
-            }
-            $qt_col = $complex_col;
-        }
-        $sql->{$stmt} .= ' ' . $qt_col;
-        return 1;
+    else {
+        $info = $ax->get_sql_info( $sql );
     }
-
+    return $info;
 }
 
 
 sub read_and_add_value {
-    my ( $sf, $sql, $clause, $stmt, $qt_col, $operator ) = @_;
+    my ( $sf, $sql, $clause, $stmt, $col, $operator, $r_data ) = @_;
     my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
     my $ext = App::DBBrowser::Table::Extensions->new( $sf->{i}, $sf->{o}, $sf->{d} );
-    my $is_numeric = $ax->is_numeric_datatype( $sql, $qt_col );
+    my $is_numeric = $ax->is_numeric_datatype( $sql, $col );
+    $r_data->[-1][-1] = $sql->{$stmt} if @{$r_data//[]};
     if ( $operator =~ /^IS\s(?:NOT\s)?NULL\z/ ) {
         return 1;
     }
@@ -185,7 +156,7 @@ sub read_and_add_value {
 
         IN: while ( 1 ) {
             # Readline
-            my $value = $ext->value( $sql, $clause, {}, $operator, { is_numeric => $is_numeric } );
+            my $value = $ext->value( $sql, $clause, $r_data, $operator, { is_numeric => $is_numeric } );
             if ( ! defined $value ) {
                 if ( ! @args ) {
                     $sql->{$stmt} = $bu_stmt;
@@ -211,18 +182,20 @@ sub read_and_add_value {
             }
             push @args, $value;
             $sql->{$stmt} = $bu_stmt . ' ('  . join ',', @args;
+            $r_data->[-1][-1] = $sql->{$stmt} if @{$r_data//[]};
         }
     }
     elsif ( $operator =~ /^(?:NOT\s)?BETWEEN\z/ ) {
         # Readline
-        my $value_1 = $ext->value( $sql, $clause, {}, $operator, { is_numeric => $is_numeric } );
+        my $value_1 = $ext->value( $sql, $clause, $r_data, $operator, { is_numeric => $is_numeric } );
         if ( ! defined $value_1 ) {
             return;
         }
         my $bu_stmt = $sql->{$stmt};
         $sql->{$stmt} .= ' ' . $value_1 . ' AND';
+        $r_data->[-1][-1] = $sql->{$stmt} if @{$r_data//[]};
         # Readline
-        my $value_2 = $ext->value( $sql, $clause, {}, $operator, { is_numeric => $is_numeric } );
+        my $value_2 = $ext->value( $sql, $clause, $r_data, $operator, { is_numeric => $is_numeric } );
         if ( ! defined $value_2 ) {
             $sql->{$stmt} = $bu_stmt;
             return;
@@ -232,7 +205,7 @@ sub read_and_add_value {
     }
     elsif ( $operator =~ /(?:REGEXP(?:_i)?|SIMILAR\sTO|MATCHES|LIKE)\z/ ) {
         # Readline
-        my $value = $ext->value( $sql, $clause, {}, $operator, { is_numeric => 0 } );
+        my $value = $ext->value( $sql, $clause, $r_data, $operator, { is_numeric => 0 } );
         if ( ! defined $value ) {
             return;
         }
@@ -244,7 +217,7 @@ sub read_and_add_value {
         }
         elsif ( $operator =~ /REGEXP(?:_i)?\z/ ) {
             if ( $sf->{i}{driver} eq 'SQLite' ) {
-                $sql->{$stmt} =~ s/ (?<=\sREGEXP\() \? (?=,\Q$qt_col\E,[01]\)\z) /$value/x;
+                $sql->{$stmt} =~ s/ (?<=\sREGEXP\() \? (?=,\Q$col\E,[01]\)\z) /$value/x;
             }
             elsif ( $sf->{i}{driver} =~ /^(?:DB2|Oracle)\z/ ) {
                 $sql->{$stmt} =~ s/ \?  (?=,'[ci]'\)\z) /$value/x;
@@ -259,8 +232,14 @@ sub read_and_add_value {
         return 1;
     }
     else {
-        # Readline
-        my $value = $ext->value( $sql, $clause, {}, $operator, { is_numeric => $is_numeric } );
+        my $value;
+        if ( $clause eq 'on' ) {
+            $value = $sf->__choose_a_column( $sql, $clause, $stmt, $r_data );
+        }
+        else {
+            # Readline
+            $value = $ext->value( $sql, $clause, $r_data, $operator, { is_numeric => $is_numeric } );
+        }
         if ( ! defined $value ) {
             return;
         }
@@ -270,7 +249,44 @@ sub read_and_add_value {
 }
 
 
-sub pattern_match {
+sub __choose_a_column {
+    my ( $sf, $sql, $clause, $stmt, $r_data ) = @_;
+    my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
+    my $tc = Term::Choose->new( $sf->{i}{tc_default} );
+    my @pre = ( undef );
+    if ( $sf->{o}{enable}{extended_cols} ) {
+        push @pre, $sf->{i}{menu_addition};
+    }
+    my @choices = @{$sql->{cols_join_condition}};
+
+    COL: while ( 1 ) {
+        my $info = $sf->info_add_condition( $sql, $clause, $stmt, $r_data );
+        # Choose
+        my $col = $tc->choose(
+            [ @pre, @choices ],
+            { %{$sf->{i}{lyt_h}}, info => $info }
+        );
+        $ax->print_sql_info( $info );
+        if ( ! defined $col ) {
+            return;
+        }
+        if ( $col eq $sf->{i}{menu_addition} ) {
+            if ( @{$r_data//[]} ) {
+                $r_data->[-1][-1] = $sql->{$stmt};
+            }
+            my $ext = App::DBBrowser::Table::Extensions->new( $sf->{i}, $sf->{o}, $sf->{d} );
+            my $complex_col = $ext->column( $sql, $clause, $r_data );
+            if ( ! defined $complex_col ) {
+                next COL;
+            }
+            $col = $complex_col;
+        }
+        return $col;
+    }
+}
+
+
+sub __pattern_match {
     my ( $sf, $col, $not_match, $case_sensitive ) = @_;
     my $driver = $sf->{i}{driver};
     if ( $driver eq 'SQLite' ) {
@@ -293,7 +309,7 @@ sub pattern_match {
     }
     elsif ( $driver eq 'Pg' ) {
         if ( $not_match ) {
-            return " ${col}::text !~*" if ! $case_sensitive;
+            return " ${col}::text !~*" if ! $case_sensitive; ##
             return " ${col}::text !~"  if   $case_sensitive;
         }
         else {
@@ -333,5 +349,4 @@ sub pattern_match {
 
 1;
 
-
 __END__
